@@ -15,12 +15,12 @@
 # Roland Giersig <RGiersig@cpan.org>
 #
 
-require 5;			# 4 won't cut it. 
+use 5;				# 4 won't cut it. 
 
 package Expect;
 
-use IO::Pty;			# This appears to require 5.004
-use IO::Stty;
+use IO::Pty 0.97;		# We need make_slave_controlling_terminal()
+use IO::Tty;
 
 use strict 'refs';
 use strict 'vars';
@@ -28,6 +28,7 @@ use strict 'subs';
 use POSIX;			# For setsid. 
 use Fcntl;			# For checking file handle settings.
 use Carp qw(cluck croak carp confess);
+use IO::Handle;
 use Exporter;
 
 # This is necessary to make routines within Expect work.
@@ -36,7 +37,7 @@ use Exporter;
 @Expect::EXPORT = qw(expect exp_continue exp_continue_timeout);
 
 BEGIN {
-  $Expect::VERSION = "1.12";
+  $Expect::VERSION = 1.14;
   # These are defaults which may be changed per object, or set as
   # the user wishes.
   # This will be unset, since the default behavior differs between 
@@ -54,16 +55,6 @@ BEGIN {
   %Expect::Spawned_PIDs = ();
 }
 
-# propagate SIGWINCH to children; but we need to also propagate terminal
-# size changes, so this is work to do.
-#$SIG{WINCH} = sub {
-#  my @pids = keys %Expect::Spawned_PIDs;
-#  print STDERR "SIGWINCH received, propagating to @pids\r\n";
-#  if (@pids) {
-#    kill WINCH => @pids;
-#  }
-#};
-
 sub version {
   my($version) = shift;
   warn "Version $version is later than $Expect::VERSION. It may not be supported" if (defined ($version) && ($version > $Expect::VERSION));
@@ -72,54 +63,95 @@ sub version {
   return $Expect::VERSION;
 }
 
-sub new { goto &spawn; }	# We should be called as Expect->spawn
-                                # or spawn Expect
+sub new {
 
-sub spawn {
-  my($tty);
-  my($name_of_tty);
   my ($class) = shift;
+  $class = ref($class) if ref($class); # so we can be called as $exp->new()
+
   # Create the pty which we will use to pass process info.
   my($self) = new IO::Pty;
-  bless ($self, $class);
-  my(@cmd) = @_;	# spawn is passed command line args.
-  ${*$self}{"exp_Command"} = \@cmd;
-  $name_of_tty = $self->IO::Pty::ttyname();
-  die "$class: Could not assign a pty" unless $name_of_tty;
-  $self->autoflush();
+  die "$class: Could not assign a pty" unless $self;
+  bless $self => $class;
+  $self->autoflush(1);
 
   # This is defined here since the default is different for
   # initialized handles as opposed to spawned processes.
   ${*$self}{exp_Log_Stdout} = 1;
   $self->_init_vars();
 
-  ${*$self}{exp_Pid} = fork;
-  unless (defined (${*$self}{exp_Pid})) {
-    warn "Cannot fork: $!";
+  if (@_) {
+    # we got add'l parms, so pass them to spawn
+    return $self->spawn(@_);
+  }
+  return $self;
+}
+
+sub spawn {
+  my ($class) = shift;
+  my $self;
+
+  if (ref($class)) {
+    $self = $class;
+  } else {
+    $self = $class->new();
+  }
+
+  my(@cmd) = @_;	# spawn is passed command line args.
+  ${*$self}{"exp_Command"} = \@cmd;
+
+  # set up pipe to detect childs exec error
+  pipe(STAT_RDR, STAT_WTR) or die "Cannot open pipe: $!";
+  STAT_WTR->autoflush(1);
+
+  my $pid = fork;
+
+  unless (defined ($pid)) {
+    warn "Cannot fork: $!" if $^W;
     return undef;
   }
-  unless (${*$self}{exp_Pid}) {
-    # Child
-    # Create a new 'session', lose controlling terminal.
-    POSIX::setsid() || warn "Couldn't perform setsid. Strange behavior may result.\r\n Problem: $!\r\n";
-    $tty = $self->IO::Pty::slave(); # Create slave handle.
-    # We have to close everything and then reopen ttyname after to get
-    # a controlling terminal.
-    close($self);
-    close STDIN; close STDOUT;
-    open(STDIN,"<&". $tty->fileno()) || die "Couldn't reopen ". $name_of_tty ." for reading, $!\r\n";
-    open(STDOUT,">&". $tty->fileno()) || die "Couldn't reopen ". $name_of_tty ." for writing, $!\r\n";
-    close STDERR; # put that here or we would never see those die's above...
-    open(STDERR,">&". $tty->fileno()) || die "Couldn't redirect STDERR, $!\r\n";
 
-    exec (@cmd);
-#    open(STDERR,">&2");
-    die "Cannot exec `@cmd': $!\n";
-    # End Child.
+  if($pid) {
+    # parent
+    my $errno;
+    ${*$self}{exp_Pid} = $pid;
+    close STAT_WTR;
+    $self->close_slave();
+    $self->set_raw() if $self->raw_pty and isatty($self);
+
+    # now wait for child exec (eof due to close-on-exit) or exec error
+    my $errstatus = sysread(STAT_RDR, $errno, 256);
+    die "Cannot sync with child: $!" if not defined $errstatus;
+    close STAT_RDR;
+    if ($errstatus) {
+      $! = $errno+0;
+      warn "Cannot exec(@cmd): $!\n" if $^W;
+      return undef;
+    }
   }
+  else {
+    # child
+    close STAT_RDR;
 
-  # Parent
-  sleep 0;
+    $self->make_slave_controlling_terminal();
+    my $slv = $self->slave()
+      or die "Cannot get slave: $!";
+
+    $slv->set_raw() if $self->raw_pty;
+    close($self);
+    close(STDIN);
+    open(STDIN,"<&". $slv->fileno())
+      or die "Couldn't reopen STDIN for reading, $!\n";
+    close(STDOUT);
+    open(STDOUT,">&". $slv->fileno())
+      or die "Couldn't reopen STDOUT for writing, $!\n";
+    close(STDERR);
+    open(STDERR,">&". $slv->fileno())
+      or die "Couldn't reopen STDERR for writing, $!\n";
+
+    { exec(@cmd) };
+    print STAT_WTR $!+0;
+    die "Cannot exec(@cmd): $!\n";
+  }
 
   # This is sort of for code compatibility, and to make debugging a little
   # easier. By code compatibility I mean that previously the process's
@@ -131,7 +163,7 @@ sub spawn {
     cluck("Spawned '@cmd'\r\n",
 	  "\t${*$self}{exp_Pty_Handle}\r\n",
 	  "\tPid: ${*$self}{exp_Pid}\r\n",
-	  "\tTty: $name_of_tty\r\n",
+	  "\tTty: ".$self->SUPER::ttyname()."\r\n",
 	 );
   }
   $Expect::Spawned_PIDs{${*$self}{exp_Pid}} = undef;
@@ -148,7 +180,7 @@ sub exp_init {
   bless $self, $class;
   croak "exp_init not passed a file object, stopped"
     unless defined($self->fileno());
-  $self->autoflush();
+  $self->autoflush(1);
   # Define standard variables.. debug states, etc.
   $self->_init_vars();
   # Turn of logging. By default we don't want crap from a file to get spewed
@@ -172,11 +204,13 @@ my %Writeable_Vars = ( debug            => 'exp_Debug',
 		       do_soft_close    => 'exp_Do_Soft_Close',
 		       max_accum        => 'exp_Max_Accum',
 		       match_max        => 'exp_Max_Accum',
+		       notransfer       => 'exp_NoTransfer',
 		       log_stdout       => 'exp_Log_Stdout',
 		       log_user         => 'exp_Log_Stdout',
 		       log_group        => 'exp_Log_Group',
 		       manual_stty      => 'exp_Manual_Stty',
 		       restart_timeout_upon_receive => 'exp_Continue',
+		       raw_pty           => 'exp_Raw_Pty',
 		     );
 my %Readable_Vars = ( pid               => 'exp_Pid',
 		      exp_pid           => 'exp_Pid',
@@ -216,11 +250,12 @@ sub AUTOLOAD {
     croak "ERROR: cannot find method `$name' in class $type";
   }
   my $varname = $Readable_Vars{$name};
-  my $tmp = ${*$self}{$varname};
+  my $tmp;
+  $tmp = ${*$self}{$varname} if exists ${*$self}{$varname};
 
   if (@_) {
     if (exists $Writeable_Vars{$name}) {
-      my $ref = ref(${*$self}{$varname});
+      my $ref = ref($tmp);
       if ($ref eq 'ARRAY') {
 	${*$self}{$varname} = [ @_ ];
       } elsif ($ref eq 'HASH') {
@@ -278,7 +313,7 @@ sub set_group {
     warn("Attempting to set a handle group on ${*$self}{exp_Pty_Handle}, ",
 	 "a non-readable handle!\r\n");
   }
-  while ($write_handle = shift) { 
+  while ($write_handle = shift) {
     if ($write_handle->_get_mode() !~ 'w') {
       warn("Attempting to set a non-writeable listen handle ",
 	   "${*$write_handle}{exp_Pty_handle} for ",
@@ -299,6 +334,7 @@ sub log_file {
 
     if (${*$self}{exp_Log_File} and ref(${*$self}{exp_Log_File}) ne 'CODE') {
       close(${*$self}{exp_Log_File});
+      ${*$self}{exp_Log_File} = undef;
     }
     return if (not $file);
     my $fh = $file;
@@ -306,41 +342,61 @@ sub log_file {
       # it's a filename
       $fh = new IO::File $file, $mode
 	or croak "Cannot open logfile $file: $!";
-    } elsif (ref($file) ne 'CODE') {
+    }
+    if (ref($file) ne 'CODE') {
       croak "Given logfile doesn't have a 'print' method"
 	if not $fh->can("print");
+      $fh->autoflush(1);		# so logfile is up to date
     }
+
     ${*$self}{exp_Log_File} = $fh;
 }
 
+
 # I'm going to leave this here in case I might need to change something.
 # Previously this was calling `stty`, in a most bastardized manner.
 sub exp_stty {
   my($self) = shift;
-  my($mode) = shift;
+  my($mode) = "@_";
+  
   return undef unless defined($mode);
+  if (not defined $INC{"IO/Stty.pm"}) {
+    carp "IO::Stty not installed, cannot change mode";
+    return undef;
+  }
+
   if (${*$self}{"exp_Debug"}) {
     print STDERR "Setting ${*$self}{exp_Pty_Handle} to tty mode '$mode'\r\n";
   }
   unless (POSIX::isatty($self)) {
-    if (${*$self}{"exp_Debug"}) {
-      print STDERR "${*$self}{exp_Pty_Handle} is not a tty. Not changing mode\r\n";
+    if (${*$self}{"exp_Debug"} or $^W) {
+      warn "${*$self}{exp_Pty_Handle} is not a tty. Not changing mode";
     }
     return '';			# No undef to avoid warnings elsewhere.
   }
-  IO::Stty::stty($self,split(/\s/,$mode));
+  IO::Stty::stty($self, split(/\s/,$mode));
 }
 
+*stty = \&exp_stty;
+
 # If we want to clear the buffer. Otherwise Accum will grow during send_slow
 # etc. and contain the remainder after matches.
 sub clear_accum {
-  my($self) = shift;
+  my ($self) = shift;
   my ($temp) = (${*$self}{exp_Accum});
   ${*$self}{exp_Accum} = '';
   # return the contents of the accumulator.
   return $temp;
 }
 
+sub set_accum {
+  my ($self) = shift;
+  my ($temp) = (${*$self}{exp_Accum});
+  ${*$self}{exp_Accum} = shift;
+  # return the contents of the accumulator.
+  return $temp;
+}
+
 ######################################################################
 # define constants for pattern subs
 sub exp_continue() { "exp_continue" }
@@ -546,6 +602,7 @@ sub _multi_expect($$@) {
  READLOOP:
   while ($exp_cont) {
     $exp_cont = 1;
+    $err = "";
     my $rmask = '';
     my $time_left = undef;
     if (defined $timeout) {
@@ -639,7 +696,8 @@ sub _multi_expect($$@) {
 	  }
 
 	  if ($exp_matched) {
-	    ${*$exp}{exp_Accum} = $after;
+	    ${*$exp}{exp_Accum} = $after
+	      unless ${*$exp}{exp_NoTransfer};
 	    print STDERR "YES!!\r\n"
 	      if ${*$exp}{exp_Exp_Internal};
 	    print STDERR ("    Before match string: `",
@@ -700,7 +758,39 @@ sub _multi_expect($$@) {
     my $nfound = select($rmask, undef, undef, $time_left);
 
     # go until we don't find something (== timeout).
-    last READLOOP unless ($nfound);
+    if (not $nfound) {
+      # No pattern, no EOF. Did we time out?
+      $err = "1:TIMEOUT";
+      foreach my $pat (@_) {
+	foreach my $exp (@{$pat->[0]}) {
+	  $before = ${*$exp}{exp_Before} = ${*$exp}{exp_Accum};
+	  next if not defined $exp->fileno(); # skip already closed
+	  ${*$exp}{exp_Error} = $err unless ${*$exp}{exp_Error};
+	}
+      }
+      print STDERR ("TIMEOUT\r\n")
+	if ($Expect::Debug or $Expect::Exp_Internal);
+      if ($timeout_hook) {
+	my $ret;
+	print STDERR ("Calling timeout function $timeout_hook...\r\n")
+	  if ($Expect::Debug or $Expect::Exp_Internal);
+	if (ref($timeout_hook) eq 'CODE') {
+	  $ret = &{$timeout_hook}($_[0]->[0]);
+	} else {
+	  if ($#{$timeout_hook} > 3) {
+	    $ret = &{$timeout_hook->[3]}($_[0]->[0],
+					 @{$timeout_hook}[4..$#{$timeout_hook}]);
+	  } else {
+	    $ret = &{$timeout_hook->[3]}($_[0]->[0]);
+	  }
+	}
+	if ($ret and $ret eq exp_continue) {
+	  $start_loop_time = time();	# restart timeout count
+	  next READLOOP;
+	}
+      }
+      last READLOOP;
+    }
 
     my @bits = split(//,unpack('b*',$rmask));
     foreach my $pat (@_) {
@@ -790,34 +880,6 @@ sub _multi_expect($$@) {
   # End READLOOP
 
   # Post loop. Do we have anything?
-  # No pattern, no EOF. Did we time out or is the process dead?
-  if (not $exp_matched and not $err) {
-    $err = "1:TIMEOUT";
-    foreach my $pat (@_) {
-      foreach my $exp (@{$pat->[0]}) {
-	$before = ${*$exp}{exp_Before} = ${*$exp}{exp_Accum};
-	next if not defined $exp->fileno(); # skip already closed
-	${*$exp}{exp_Error} = $err unless ${*$exp}{exp_Error};
-      }
-    }
-    print STDERR ("TIMEOUT\r\n")
-      if ($Expect::Debug or $Expect::Exp_Internal);
-    if ($timeout_hook) {
-      print STDERR ("Calling timeout function $timeout_hook...\r\n")
-	if ($Expect::Debug or $Expect::Exp_Internal);
-      if (ref($timeout_hook) eq 'CODE') {
-	&{$timeout_hook}($_[0]->[0]);
-      } else {
-	if ($#{$timeout_hook} > 3) {
-	  &{$timeout_hook->[3]}($_[0]->[0],
-				@{$timeout_hook}[4..$#{$timeout_hook}]);
-	} else {
-	  &{$timeout_hook->[3]}($_[0]->[0]);
-	}
-      }
-    }
-  }
-
   # Tell us status
   if ($Expect::Debug or $Expect::Exp_Internal) {
       if ($exp_matched) {
@@ -1041,9 +1103,13 @@ sub interconnect {
   while (1) {
     # test each handle to see if it's still alive.
     foreach $read_handle (@handles) {
-      waitpid(${*$read_handle}{exp_Pid}, WNOHANG) if (defined (${*$read_handle}{exp_Pid}) &&${*$read_handle}{exp_Pid});
-      if (defined(${*$read_handle}{exp_Pid}) &&(${*$read_handle}{exp_Pid}) &&(! kill(0,${*$read_handle}{exp_Pid}))) {
-	print STDERR "Got EOF (${*$read_handle}{exp_Pty_Handle} died) reading ${*$read_handle}{exp_Pty_Handle}\r\n"if ${*$read_handle}{"exp_Debug"};
+      waitpid(${*$read_handle}{exp_Pid}, WNOHANG)
+	if (exists (${*$read_handle}{exp_Pid}) and ${*$read_handle}{exp_Pid});
+      if (exists(${*$read_handle}{exp_Pid})
+	  and (${*$read_handle}{exp_Pid})
+	  and (! kill(0,${*$read_handle}{exp_Pid}))) {
+	print STDERR "Got EOF (${*$read_handle}{exp_Pty_Handle} died) reading ${*$read_handle}{exp_Pty_Handle}\r\n"
+	  if ${*$read_handle}{"exp_Debug"};
 	last CONNECT_LOOP unless defined(${${*$read_handle}{exp_Function}}{"EOF"});
 	last CONNECT_LOOP unless &{${${*$read_handle}{exp_Function}}{"EOF"}}(@{${${*$read_handle}{exp_Parameters}}{"EOF"}});
       }
@@ -1051,8 +1117,8 @@ sub interconnect {
 
     # Every second? No, go until we get something from someone.
     ($nfound) = select($rout = $read_mask, undef, $eout = $emask, undef);
-    # Is there anything to share?
-    next CONNECT_LOOP unless $nfound;
+    # Is there anything to share?  May be -1 if interrupted by a signal...
+    next CONNECT_LOOP if not defined $nfound or $nfound < 1;
     # Which handles have stuff?
     @bits = split(//,unpack('b*',$rout));
     $eout = 0 unless defined ($eout);
@@ -1131,6 +1197,18 @@ sub interconnect {
   }
 }
 
+# user can decide if log output gets also sent to logfile
+sub print_log_file {
+  my $self = shift;
+  if (${*$self}{exp_Log_File}) {
+    if (ref(${*$self}{exp_Log_File}) eq 'CODE') {
+      ${*$self}{exp_Log_File}->(@_);
+    } else {
+      ${*$self}{exp_Log_File}->print(@_);
+    }
+  }
+}
+
 # we provide our own print so we can debug what gets sent to the
 # processes...
 sub print (@) {
@@ -1139,16 +1217,13 @@ sub print (@) {
     my $args = _make_readable(join('', @args));
     cluck "Sending '$args' to ${*$self}{exp_Pty_Handle}\r\n";
   }
-  print STDOUT @args
-    if ${*$self}{exp_Log_Stdout};
-  if (${*$self}{exp_Log_File}) {
-    if (ref(${*$self}{exp_Log_File}) eq 'CODE') {
-      ${*$self}{exp_Log_File}->(@args);
-    } else {
-      ${*$self}{exp_Log_File}->print(@args)
+  foreach my $arg (@args) {
+    while (length($arg) > 80) {
+      $self->SUPER::print(substr($arg, 0, 80));
+      $arg = substr($arg, 80);
     }
+    $self->SUPER::print($arg);
   }
-  $self->SUPER::print(@args);
 }
 
 # make an alias for Tcl/Expect users for a DWIM experience...
@@ -1255,18 +1330,20 @@ sub soft_close {
     print STDERR "${*$self}{exp_Pty_Handle} closed.\r\n";
   }
   # quit now if it isn't a process.
-  return $close_status unless defined($self->pid());
+  return $close_status unless defined(${*$self}{exp_Pid});
   # Now give it 15 seconds to die.
   $end_time = time() + 15;
   while ($end_time > time()) {
-    $returned_pid = waitpid($self->pid(), &WNOHANG);
+    $returned_pid = waitpid(${*$self}{exp_Pid}, &WNOHANG);
     # Stop here if the process dies.
     if (defined($returned_pid) && $returned_pid) {
       delete $Expect::Spawned_PIDs{$returned_pid};
       if (${*$self}{exp_Debug}) {
 	print STDERR "Pid ${*$self}{exp_Pid} of ${*$self}{exp_Pty_Handle} exited, Status: $?\r\n";
       }
-      return $?;
+      ${*$self}{exp_Pid} = undef;
+      ${*$self}{exp_Exit} = $?;
+      return ${*$self}{exp_Exit};
     }
     sleep 1;			# Keep loop nice.
   }
@@ -1278,12 +1355,14 @@ sub soft_close {
   # Now to be anal retentive.. wait 15 more seconds for it to die.
   $end_time = time() + 15;
   while ($end_time > time()) {
-    $returned_pid = waitpid($self->pid(), &WNOHANG);
+    $returned_pid = waitpid(${*$self}{exp_Pid}, &WNOHANG);
     if (defined($returned_pid) && $returned_pid) {
       delete $Expect::Spawned_PIDs{$returned_pid};
       if (${*$self}{exp_Debug}) {
 	print STDERR "Pid ${*$self}{exp_Pid} of ${*$self}{exp_Pty_Handle} terminated, Status: $?\r\n";
       }
+      ${*$self}{exp_Pid} = undef;
+      ${*$self}{exp_Exit} = $?;
       return $?;
     }
     sleep 1;
@@ -1304,18 +1383,20 @@ sub hard_close {
     print STDERR "${*$self}{exp_Pty_Handle} closed.\r\n";
   }
   # Return now if handle.
-  return $close_status unless defined($self->pid());
+  return $close_status unless defined(${*$self}{exp_Pid});
   # Now give it 5 seconds to die. Less patience here if it won't die.
   $end_time = time() + 5;
   while ($end_time > time()) {
-    $returned_pid = waitpid($self->pid(), &WNOHANG);
+    $returned_pid = waitpid(${*$self}{exp_Pid}, &WNOHANG);
     # Stop here if the process dies.
     if (defined($returned_pid) && $returned_pid) {
       delete $Expect::Spawned_PIDs{$returned_pid};
       if (${*$self}{exp_Debug}) {
 	print STDERR "Pid ${*$self}{exp_Pid} of ${*$self}{exp_Pty_Handle} exited, Status: $?\r\n";
       }
-      return $?;
+      ${*$self}{exp_Pid} = undef;
+      ${*$self}{exp_Exit} = $?;
+      return ${*$self}{exp_Exit};
     }
     sleep 1;			# Keep loop nice.
   }
@@ -1327,13 +1408,15 @@ sub hard_close {
   # wait 15 more seconds for it to die.
   $end_time = time() + 15;
   while ($end_time > time()) {
-    $returned_pid = waitpid($self->pid(), &WNOHANG);
+    $returned_pid = waitpid(${*$self}{exp_Pid}, &WNOHANG);
     if (defined($returned_pid) && $returned_pid) {
       delete $Expect::Spawned_PIDs{$returned_pid};
       if (${*$self}{exp_Debug}) {
 	print STDERR "Pid ${*$self}{exp_Pid} of ${*$self}{exp_Pty_Handle} terminated, Status: $?\r\n";
       }
-      return $?;
+      ${*$self}{exp_Pid} = undef;
+      ${*$self}{exp_Exit} = $?;
+      return ${*$self}{exp_Exit};
     }
     sleep 1;
   }
@@ -1341,17 +1424,20 @@ sub hard_close {
   # wait 5 more seconds for it to die.
   $end_time = time() + 5;
   while ($end_time > time()) {
-    $returned_pid = waitpid($self->pid(), &WNOHANG);
+    $returned_pid = waitpid(${*$self}{exp_Pid}, &WNOHANG);
     if (defined($returned_pid) && $returned_pid) {
       delete $Expect::Spawned_PIDs{$returned_pid};
       if (${*$self}{exp_Debug}) {
 	print STDERR "Pid ${*$self}{exp_Pid} of ${*$self}{exp_Pty_Handle} killed, Status: $?\r\n";
       }
-      return $?;
+      ${*$self}{exp_Pid} = undef;
+      ${*$self}{exp_Exit} = $?;
+      return ${*$self}{exp_Exit};
     }
     sleep 1;
   }
   warn "Pid ${*$self}{exp_Pid} of ${*$self}{exp_Pty_Handle} is HUNG.\r\n";
+  ${*$self}{exp_Pid} = undef;
   return undef;
 }
 
@@ -1376,6 +1462,7 @@ sub _init_vars {
   # Initialize accumulator.
   ${*$self}{exp_Max_Accum} = $Expect::Exp_Max_Accum;
   ${*$self}{exp_Accum} = '';
+  ${*$self}{exp_NoTransfer} = 0;
 
   # create empty expect_before & after lists
   ${*$self}{exp_expect_before_list} = [];
@@ -1442,14 +1529,8 @@ sub _print_handles {
   # If ${*$self}{exp_Pty_Handle} is STDIN this would make it echo.
   print STDOUT $print_this
     if ${*$self}{"exp_Log_Stdout"};
-  if (${*$self}{exp_Log_File}) {
-    if (ref(${*$self}{exp_Log_File}) eq 'CODE') {
-      ${*$self}{exp_Log_File}->($print_this);
-    } else {
-      ${*$self}{exp_Log_File}->print($print_this)
-    }
-  }
-  $|= 1;			# This should not be necessary but autoflush() doesn't always work.
+  $self->print_log_file($print_this);
+  $|= 1; # This should not be necessary but autoflush() doesn't always work.
 }
 
 sub _get_mode {
